/*
 * This file is a part of AllPlaceFinder library
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2021-2024, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef headerfile_allplacefinder_lib_finder
#define headerfile_allplacefinder_lib_finder

#include <fstream>
#include "html/htmlparser.h"
#include "params.h"


namespace allplacefinder
{


class Finder : public pt::HTMLParser::Listener
{
public:

	Finder();

	bool create_lookup_dir(Params & params);
	bool create_hints_file(Params & params);
	void clear();


private:

	std::ifstream osm_file_str;
	std::ifstream lookup_dir_str;
	std::ofstream hints_file;
	Params * params;
	pt::TextStream text_stream;
	size_t nodes;
	size_t ways;
	bool second_phase;
	std::map<long, pt::Space> ways_table;

	bool parse_osm_file();
	void item_parsed(const pt::HTMLParser::Item & item) override;
	bool should_remove(const pt::HTMLParser::Item & item) override;
	void parse_node(pt::Space & node);
	void parse_way(pt::Space & way);
	bool parse_way_childs(pt::Space & out, pt::Space & childs);
	bool parse_way_name(pt::Space & out, pt::Space & childs, size_t & nd_len);
	bool parse_way_middle_node(pt::Space & out, pt::Space & childs, size_t nd_len);
	bool save_file(const wchar_t * subdir, long id, pt::Space & space);
	bool get_node(long node_id, pt::Space & out);
	bool check_way(long node_id, std::wstring & lat, std::wstring & lon);

};

} // namespace

#endif
